#include "onedimensionnewton.h"

OneDimensionNewton::OneDimensionNewton(Problem *p)
{
    myProblem = p;
    maxiters = 100;
}

double OneDimensionNewton::estimateSecondDerivative(double x)
{
    const double h = 1e-6;
    Data xx;xx.resize(1);
    xx[0]=x;
    xx[0]=x+h;
    double y1=myProblem->funmin(xx);
    xx[0]=x;
    double y2=myProblem->funmin(xx);
    xx[0]=x-h;
    double y3 =myProblem->funmin(xx);
    return (y1-2*y2+y3)/(h * h);
}

void    OneDimensionNewton::setMaxiters(int i)
{
   if(i>0) maxiters =i;
}

int    OneDimensionNewton::getMaxiters() const
{
    return maxiters;
}

bool   OneDimensionNewton::Terminated()
{
   ++iter;
    printf("NEWTON. Iter=%4d F(x)=%20.10lg\n",iter,ypoint);
    return iter>=maxiters || myProblem->grms(xpoint)<1e-4;
}

void   OneDimensionNewton::Step()
{
    Data g = myProblem->gradient(xpoint);
    double y1 = g[0];
    double y2 = estimateSecondDerivative(xpoint[0]);
    xpoint[0]=xpoint[0]-y1/y2;
    ypoint = myProblem->statFunmin(xpoint);
}

void   OneDimensionNewton::Solve()
{
    xpoint = myProblem->getSample();
    ypoint = myProblem->statFunmin(xpoint);
    iter = 0;
    do
    {
        Step();
    }while(!Terminated());
}

Data   OneDimensionNewton::getBestx()
{
    return xpoint;
}

double OneDimensionNewton::getBesty()
{
    return ypoint;
}

OneDimensionNewton::~OneDimensionNewton()
{

}
